@@ -449,22 +449,30 @@ set_track_option_menu (GtkOptionMenu *menu,
 					   G_CALLBACK (skip_to_track), NULL);
 }
 
+/* set the window title
+ *   with the current artist (through gcd)
+ *   and the track title (through status' track index)
+ * FIXME: only called during PAUSE/PLAY state changes, while this
+ *        would make sense to be used during track changes too.
+ *        What is being done instead there ?
+ */
 static void
 set_window_track_title (GnomeCD *gcd,
 			GnomeCDRomStatus *status)
 {
 	int idx = status->track - 1;
-	const char *artist = "";
-	const char *track_name = "";
+	const char *artist = NULL;
+	const char *track_name = NULL;
 
 	if (gcd->disc_info) {
 		if (idx >= 0 && idx < gcd->disc_info->ntracks &&
 		    gcd->disc_info->track_info)
 			track_name = gcd->disc_info->track_info [idx]->name;
 
-		artist = gcd->disc_info->artist ? gcd->disc_info->artist : "";
+		artist = gcd->disc_info->artist;
 	}
 
+	gcd_debug ("track_title: setting artist %s, track_name %s", artist, track_name);
 	gnome_cd_set_window_title (gcd, artist, track_name);
 }
 
@@ -483,8 +491,7 @@ status_ok (GnomeCD *gcd,
 	/* Set the track menu on. */
 	gtk_widget_set_sensitive (gcd->tracks, TRUE);
 	
-	/* All buttons can be used when the tray is open,
-	   they should shut the tray. */
+	/* All buttons can be used when the state is ok */
 	gtk_widget_set_sensitive (gcd->rewind_b, TRUE);
 	gtk_widget_set_sensitive (gcd->play_b, TRUE);
 	gtk_widget_set_sensitive (gcd->stop_b, TRUE);
@@ -508,8 +515,11 @@ status_ok (GnomeCD *gcd,
 			gcd->current_image = gcd->pause_image;
 		}
 		/* Find out if the track has changed */
-		track = gtk_option_menu_get_history (GTK_OPTION_MENU (gcd->tracks));
-		if (track + 1 != status->track) {
+                /* FIXME: this would be a good place to call
+                   set_window_track_title instead.
+                   Also, gtk_option_menu_get_history is now deprecated. */
+		track = gtk_option_menu_get_history (GTK_OPTION_MENU (gcd->tracks)) + 1;
+		if (track != status->track) {
 			set_track_option_menu (GTK_OPTION_MENU (gcd->tracks),
 					       status->track);
 		}
@@ -625,11 +635,10 @@ status_ok (GnomeCD *gcd,
 		cd_display_set_line (CD_DISPLAY (gcd->display),
 				     CD_DISPLAY_LINE_TIME, "");
 		if (gcd->disc_info != NULL) {
+                        gcd_debug ("AUDIO_STOP: artist %s, title %s", gcd->disc_info->artist, gcd->disc_info->title);
 			gnome_cd_set_window_title (gcd,
-						   gcd->disc_info->artist ?
-						   gcd->disc_info->artist : "",
-						   gcd->disc_info->title ?
-						   gcd->disc_info->title : "");
+						   gcd->disc_info->artist,
+						   gcd->disc_info->title);
 		} else {
 			gnome_cd_set_window_title (gcd, NULL, NULL);
 		}
@@ -644,10 +653,9 @@ status_ok (GnomeCD *gcd,
 		cd_display_set_line (CD_DISPLAY (gcd->display),
 				     CD_DISPLAY_LINE_TIME, _("Disc error"));
 		if (gcd->disc_info != NULL) {
-			gnome_cd_set_window_title (gcd, gcd->disc_info->artist ?
-						   gcd->disc_info->artist : "",
-						   gcd->disc_info->title ?
-						   gcd->disc_info->title : "");
+                        gcd_debug ("AUDIO_ERROR: artist %s, title %s", gcd->disc_info->artist, gcd->disc_info->title);
+			gnome_cd_set_window_title (gcd, gcd->disc_info->artist,
+						   gcd->disc_info->title);
 		} else {
 			gnome_cd_set_window_title (gcd, NULL, NULL);
 		}
@@ -659,11 +667,10 @@ status_ok (GnomeCD *gcd,
 		
 	default:
 		if (gcd->disc_info != NULL) {
+                        gcd_debug ("default: artist %s, title %s", gcd->disc_info->artist, gcd->disc_info->title);
 			gnome_cd_set_window_title (gcd,
-						   gcd->disc_info->artist ?
-						   gcd->disc_info->artist : "",
-						   gcd->disc_info->title ?
-						   gcd->disc_info->title : "");
+						   gcd->disc_info->artist,
+						   gcd->disc_info->title);
 		} else {
 			gnome_cd_set_window_title (gcd, NULL, NULL);
 		}
@@ -698,6 +705,7 @@ status_ok (GnomeCD *gcd,
 	}
 }
 
+/* GnomeCDRom status-changed signal handler */
 void
 cd_status_changed_cb (GnomeCDRom *cdrom,
 		      GnomeCDRomStatus *status,
@@ -706,20 +714,28 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 	if (gcd->not_ready == TRUE) {
 		return;
 	}
-	
+
 	switch (status->cd) {
 	case GNOME_CDROM_STATUS_OK:
+                /* does everything when the state is ok */
 		status_ok (gcd, status);
-		if (status->track >= gcd->disc_info->ntracks)
+                /* next is sensitive when this is not the last track */
+		/* but gcd->disc_info might be NULL when
+                 * this gets called on startup but we haven't looked at the
+                 * disc yet */
+		if (gcd->disc_info && status->track >= gcd->disc_info->ntracks)
                        gtk_widget_set_sensitive (gcd->next_b, FALSE);
 		else
                        gtk_widget_set_sensitive (gcd->next_b, TRUE);
+                /* back is sensitive when it's not the first track and
+                   we're playing */
                if (status->track <= 1 && (status->audio == GNOME_CDROM_AUDIO_STOP || status->audio == GNOME_CDROM_AUDIO_COMPLETE))
                        gtk_widget_set_sensitive (gcd->back_b, FALSE);
 		else
-                       gtk_widget_set_sensitive (gcd->back_b, FALSE);
+                       gtk_widget_set_sensitive (gcd->back_b, TRUE);
 		break;
-		
+
+        /* everything below are states where we're not playing */
 	case GNOME_CDROM_STATUS_NO_DISC:
 		if (gcd->disc_info != NULL) {
 			cddb_free_disc_info (gcd->disc_info);
@@ -749,7 +765,7 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		if (gcd->disc_info != NULL) {
 			cddb_free_disc_info (gcd->disc_info);
 			gcd->disc_info = NULL;
-			//we need to destroy the hashtable(cddb_cache) 
+			/* we need to destroy the hashtable(cddb_cache) */
 			destroy_cache_hashTable();
 		}
 		
